import os
import inspect
from jinja2 import ChoiceLoader, FileSystemLoader, PackageLoader
from run import Module, NullTask, RenderTask, DerivedTask, FindVar, ValueVar
# TODO: Nosetests import issue
from run.module import module

class ManageModule(Module):

    # Public

    def __init__(self):
        self._update_pythonpath()

    # Meta

    meta_strict = False

    # Tasks

    build = NullTask(
        require=[
            'build_gitignore',
            'build_travis',
            'build_manifest',
            'build_readme',
            'build_requirements',
            'build_setup',
            'docs.build',
        ],
    )

    build_gitignore = DerivedTask(
        task='render',
        source='.gitignore',
        target='.gitignore',
    )

    build_travis = DerivedTask(
        task='render',
        source='.travis.yml',
        target='.travis.yml',
    )

    build_manifest = DerivedTask(
        task='render',
        source='MANIFEST.in',
        target='MANIFEST.in',
    )

    build_readme = DerivedTask(
        task='render',
        source='README.rst',
        target='README.rst',
    )

    build_requirements = DerivedTask(
        task='render',
        source='requirements.txt',
        target='requirements.txt',
    )

    build_setup = DerivedTask(
        task='render',
        source='setup.py',
        target='setup.py',
    )

    render = RenderTask(
        trim_blocks=True,
        lstrip_blocks=True,
        loader=ChoiceLoader([
            FileSystemLoader('_sources'),
            PackageLoader('packgram', '_sources'),
        ]),
    )

    # Vars

    author = '<author>'
    author_email = '<author_email>'
    caution = 'DO NOT CHANGE THIS FILE. SOURCE IS IN "_sources" DIRECTORY.'
    classifiers = []
    data_files = []
    description = '<description>'
    development_requires = []
    entry_points = {}
    install_requires = []
    license = '<license>'
    name = '<name>'
    platforms = []
    pypi_password_secure = '<pypi_password_secure>'
    tests_require = []
    test_suite = '<test_suite>'

    long_description = FindVar(
        filepath='README.rst',
        getfirst=True,
    )

    github_user = ValueVar(
        value=module().author,
    )

    maintainer = ValueVar(
        value=module().author,
    )

    maintainer_email = ValueVar(
        value=module().author_email,
    )

    pypi_name = ValueVar(
        value=module().name,
    )

    pypi_user = ValueVar(
        value=module().author,
    )

    version = FindVar(
        mode='objects',
        objname='version',
        filepath='*/version.py',
        getfirst=True,
    )

    # Protected

    def _update_pythonpath(self):
        os.environ['PYTHONPATH'] = ':'.join([
            os.environ.get('PYTHONPATH', ''),
            os.path.abspath(os.path.dirname(inspect.getfile(type(self))))])
